#include "MotionSequenceMC.h"
#include "Shared/get_time.h"
#include "Shared/WorldState.h"
#include <iostream>

using std::cout;
using std::endl;

MotionSequence::Move_idx_t MotionSequence::invalid_move=-1U;

int MotionSequence::updateJointCmds() {
	if(isPlaying()) {
		if(lasttime==0)
			play();
		double diff=(get_time()-lasttime)*playspeed;
		if(diff+playtime<0)
			setPlayTime(0);
		else
			setPlayTime(static_cast<unsigned int>(diff+playtime));
		lasttime=get_time();
		return 1;
	} else {
		lasttime=get_time();
		return 0;
	}
}

const JointCmd& MotionSequence::getJointCmd(unsigned int i) {
	if(curstamps[i]!=playtime) {
		if(nexts[i]!=invalid_move)
			calcJoint(curs[i],playtime,getKeyFrame(prevs[i]),getKeyFrame(nexts[i]));
		else
			curs[i]=unusedJoint;
		curstamps[i]=playtime;
	}
	return curs[i];
}

unsigned int MotionSequence::getBinSize() const {
	char buf[128];
	unsigned int len=128;
	unsigned int used=strlen("#MSq\n");
	unsigned int t=0;
	Move_idx_t tprevs[NumOutputs];
	Move_idx_t tnexts[NumOutputs];
	for(unsigned int i=0;i<NumOutputs;i++)
		tnexts[i]=getKeyFrame(tprevs[i]=starts[i]).next;
	while(t!=-1U) {
		for(unsigned int i=0; i<NumOutputs; i++) {
			if(getKeyFrame(tprevs[i]).cmd.weight!=0 && getKeyFrame(tprevs[i]).starttime==t) {
				if(getKeyFrame(tprevs[i]).cmd.weight==1) {
					if(t!=0)
						used+=snprintf(buf,len,"%s\t%g\n",outputNames[i],getKeyFrame(tprevs[i]).cmd.value);
				} else
					used+=snprintf(buf,len,"%s\t%g\t%g\n",outputNames[i],getKeyFrame(tprevs[i]).cmd.value,getKeyFrame(tprevs[i]).cmd.weight);
			}
		}
		unsigned int last=t;
		t=setNextFrameTime(tprevs,tnexts);
		if(t!=-1U)
			used+=snprintf(buf,len,"delay\t%d\n",t-last);
	}
	used+=strlen("#END\n");
	return used+1;
}

unsigned int MotionSequence::LoadBuffer(const char buf[], unsigned int len) {
	unsigned int origlen=len;
	if(strncmp("#MSq\n",buf,5)!=0) {
		//		cout << "ERROR MotionSequence load corrupted - expected #MSq header" << endl;
		return 0;
	}
	unsigned int linenum=2;
	unsigned int lastJointIdx=0;
	while(len<=origlen && len>0) {
		int written;
		//		printf("%d %.9s\n",linenum+1,buf);
		if(buf[0]=='#' || buf[0]=='\n') {
			if(strncmp("#END\n",buf,5)==0)
				return origlen-len;
			else {
				while(*buf++!='\n') {}
				continue;
			}
		}
		written=-1;
		const unsigned int cmdlen=16, arglen=32;
		char command[cmdlen];
		char arg1[arglen];
		char arg2[arglen];
		written=readWord(buf,&buf[len],command,cmdlen);
		if(!ChkAdvance(written,&buf,&len,"*** ERROR MotionSequence load corrupted - line %d\n",linenum)) return 0;
		written=readWord(buf,&buf[len],arg1,arglen);
		if(!ChkAdvance(written,&buf,&len,"*** ERROR MotionSequence load corrupted - line %d\n",linenum)) return 0;
		written=readWord(buf,&buf[len],arg2,cmdlen);
		if(written!=0)
			if(!ChkAdvance(written,&buf,&len,"*** ERROR MotionSequence load corrupted - line %d\n",linenum)) return 0;
		for(;len>0 && *buf!='\n';buf++,len--) {}

		if(strcasecmp(command,"delay")==0) {
			char* used;
			int delay = strtol(arg1,&used,0);
			if(*used!='\0') {
				cout << "*** WARNING illegal delay argument: " << arg1 << " - line " << linenum << endl;
			} else {
				setPlayTime(playtime+delay);
			}
		} else if(strcasecmp(command,"settime")==0) {
			char* used;
			int newtime = strtol(arg1,&used,0);
			if(*used!='\0') {
				cout << "*** WARNING illegal settime argument: " << arg1 << " - line " << linenum << endl;
			} else {
				setPlayTime(newtime);
			}
		} else if(strcasecmp(command,"load")==0) {
			PostureEngine pose;
			std::string f;
			if(arg1[0]!='/')
				f="/ms/data/motion/";
			f+=arg1;
			if(pose.LoadFile(f.c_str())!=0)
				setPose(pose);
			else
				cout << "*** WARNING could not read file " << arg1 << " for load - line " << linenum << endl;
		} else if(strcasecmp(command,"overlay")==0) {
			PostureEngine pose;
			std::string f;
			if(arg1[0]!='/')
				f="/ms/data/motion/";
			f+=arg1;
			if(pose.LoadFile(f.c_str())!=0)
				overlayPose(pose);
			else if(LoadFile(f.c_str())==0)
				cout << "*** WARNING could not read file " << arg1 << " for overlay - line " << linenum << endl;
		} else {
			lastJointIdx=getJointIndex(command,lastJointIdx+1);
			if(lastJointIdx==NumOutputs)
				cout << "*** WARNING illegal command or jointName: " << command << endl;
			else {
				char* used;
				double value=strtod(arg1,&used), weight=1;
				if(*used!='\0')
					cout << "*** WARNING illegal value argument: " << arg1 << " - line " << linenum << endl;
				else if(arg2[0]!='\0') {
					weight=strtod(arg2,&used);
					if(*used!='\0') {
						cout << "*** WARNING illegal weight argument: " << arg2 << " - line " << linenum << endl;
						weight=1;
					}
					setJointCmd(lastJointIdx,JointCmd(value,weight));
				}
			}
		}

		linenum++;

	}
	cout << "*** WARNING MotionSequence load missing #END" << endl;
	return origlen-len;
}

unsigned int MotionSequence::SaveBuffer(char buf[], unsigned int len) const {
	cout << "SAVEBUFFER..." << flush;
	unsigned int origlen=len;
	int written=snprintf(buf,len,"#MSq\n");
	if(!ChkAdvance(written,(const char**)&buf,&len,"*** ERROR MotionSequence save failed on header\n")) return 0;	if(len==0 || len>origlen) {
		cout << "*** ERROR MotionSequence save overflow on header" << endl;
		return 0;
	}
	unsigned int t=0;
	Move_idx_t tprevs[NumOutputs];
	Move_idx_t tnexts[NumOutputs];
	for(unsigned int i=0;i<NumOutputs;i++)
		tnexts[i]=getKeyFrame(tprevs[i]=starts[i]).next;
	while(t!=-1U) {
		cout << "t="<<t<<"..."<<flush;
		for(unsigned int i=0; i<NumOutputs; i++) {
			if(getKeyFrame(tprevs[i]).cmd.weight!=0 && getKeyFrame(tprevs[i]).starttime==t) {
				if(getKeyFrame(tprevs[i]).cmd.weight==1) {
					if(t!=0) {
						written=snprintf(buf,len,"%s\t%g\n",outputNames[i],getKeyFrame(tprevs[i]).cmd.value);
						if(!ChkAdvance(written,(const char**)&buf,&len,"*** ERROR MotionSequence save failed\n")) return 0;
					}
				} else {
					written=snprintf(buf,len,"%s\t%g\t%g\n",outputNames[i],getKeyFrame(tprevs[i]).cmd.value,getKeyFrame(tprevs[i]).cmd.weight);
					if(!ChkAdvance(written,(const char**)&buf,&len,"*** ERROR MotionSequence save failed\n")) return 0;
				}
				if(len==0 || len>origlen) {
					cout << "*** ERROR MotionSequence save overflow" << endl;
					return 0;
				}
			}
		}
		unsigned int last=t;
		t=setNextFrameTime(tprevs,tnexts);
		if(t!=-1U) {
			written=snprintf(buf,len,"delay\t%d\n",t-last);
			if(!ChkAdvance(written,(const char**)&buf,&len,"*** ERROR MotionSequence save failed\n")) return 0;
			if(len==0 || len>origlen) {
				cout << "*** ERROR MotionSequence save overflow" << endl;
				return 0;
			}
		}
	}
	written=snprintf(buf,len,"#END\n");
	if(!ChkAdvance(written,(const char**)&buf,&len,"*** ERROR MotionSequence save failed on #END\n")) return 0;
	if(len==0 || len>origlen) {
		cout << "*** ERROR MotionSequence save overflow on #END" << endl;
		return 0;
	}
	return origlen-len;
	cout << "SAVE-done!" << endl;
}

void MotionSequence::setPlayTime(unsigned int x) {
	playtime=x;
	for(unsigned int i=0; i<NumOutputs; i++)
		setRange(x,prevs[i],nexts[i]);
}

void MotionSequence::setJointCmd(unsigned int i, const JointCmd& cmd) {
	Move& prev_m=getKeyFrame(prevs[i]);
	if(playtime==prev_m.starttime) { //edit current keyframe
		prev_m.cmd=cmd;
	} else { //add new keyframe
		Move_idx_t x = newKeyFrame();
		Move& cur=getKeyFrame(x);
		cur.cmd=cmd;
		cur.starttime=playtime;
		cur.prev=prevs[i];
		cur.next=nexts[i];
		prev_m.next=x;
		if(nexts[i]!=invalid_move) //in middle
			getKeyFrame(nexts[i]).prev=x;
		else { //at end
			if(playtime>endtime)
				endtime=playtime;
		}
		prevs[i]=x;
	}
}

void MotionSequence::setPose(const PostureEngine& pose) {
	for(unsigned int i=0; i<NumOutputs; i++)
		setJointCmd(i,pose.getJointCmd(i));
}

void MotionSequence::overlayPose(const PostureEngine& pose) {
	for(unsigned int i=0; i<NumOutputs; i++)
		if(pose.getJointCmd(i)!=unusedJoint)
			setJointCmd(i,pose.getJointCmd(i));
}

void MotionSequence::compress() {
	for(unsigned int i=0; i<NumOutputs; i++) {
		Move_idx_t prev=getKeyFrame(starts[i]).next;
		if(prev==(Move_idx_t)-1)
			break;
		Move_idx_t cur=getKeyFrame(prev).next;
		if(cur==(Move_idx_t)-1)
			break;
		Move_idx_t next=getKeyFrame(cur).next;
		while(next!=(Move_idx_t)-1) {
			JointCmd tmp;
			Move& prev_m=getKeyFrame(prev);
			Move& cur_m=getKeyFrame(cur);
			Move& next_m=getKeyFrame(next);
			calcJoint(tmp,cur_m.starttime,prev_m,next_m);
			if(tmp==cur_m.cmd || tmp.weight==0 && cur_m.cmd.weight==0) {
				prev_m.next=next;
				next_m.prev=prev;
				eraseKeyFrame(cur);
			} else
				prev=cur;
			cur=next;
			next=next_m.next;
		}
	}
}

void MotionSequence::play() {
	if(playspeed>0)
		setPlayTime(0);
	else
		setPlayTime(endtime);
	resume();
}

void MotionSequence::resume() {
	playing=true;
	lasttime=get_time();
	for(unsigned int i=0; i<NumOutputs; i++) {
		Move_idx_t cur=starts[i];
		while(cur!=(Move_idx_t)-1) {
			if(getKeyFrame(cur).cmd.weight!=0) {
				getKeyFrame(starts[i]).cmd.value=state->outputs[i];
				break;
			}
			cur=getKeyFrame(cur).next;
		}
	}
}

bool MotionSequence::ChkAdvance(int res, const char** buf, unsigned int* len, const char* msg) {
	if(res>0) {
		*buf+=res;
		*len-=res;
		return true;
	} else {
		printf("%s",msg);
		return false;
	}
}

bool MotionSequence::ChkAdvance(int res, const char** buf, unsigned int* len, const char* msg, int arg1) {
	if(res>0) {
		*buf+=res;
		*len-=res;
		return true;
	} else {
		printf(msg,arg1);
		return false;
	}
}

unsigned int MotionSequence::setNextFrameTime(Move_idx_t p[NumOutputs], Move_idx_t n[NumOutputs]) const {
	unsigned int ans=-1U;
	for(unsigned int i=0; i<NumOutputs; i++)
		if(n[i]!=invalid_move && getKeyFrame(n[i]).starttime<ans)
			ans=getKeyFrame(n[i]).starttime;
	if(ans!=-1U)
		for(unsigned int i=0; i<NumOutputs; i++)
			setRange(ans,p[i],n[i]);
	return ans;
}

unsigned int MotionSequence::readWord(const char buf[], const char * const bufend, char wrd[], const unsigned int wordlen) {
	const char* origbuf=buf;
	wrd[0]='\0';
	unsigned int i;
	//skip whitespace
	for(;buf<bufend && isspace(*buf) && *buf!='\n';buf++) {}
	//store wrd
	for(i=0; buf<bufend && !isspace(*buf); buf++)
		if(i<wordlen-1)
			wrd[i++]=*buf;
	wrd[i]='\0';
	if(buf>=bufend)
		return -1U;
	return buf-origbuf;
}

unsigned int MotionSequence::getJointIndex(const char name[], unsigned int idx) {
	unsigned int startidx=idx;
	for(;idx<NumOutputs;idx++)
		if(strcmp(name,outputNames[idx])==0) {
			//				cmds[idx].set(fval,fwht);
			break;
		}
	if(idx==NumOutputs) {
		for(idx=0;idx<startidx;idx++)
			if(strcmp(name,outputNames[idx])==0) {
				//					cmds[idx].set(fval,fwht);
				break;
			}
		if(idx==startidx && strcmp(name,outputNames[idx])!=0)
			return NumOutputs;
	}
	return idx;
}

/*! @file
 * @brief Implements MotionSequence, handy little (or not so little) class for switching between a sequence of postures
 * @author ejt (Creator)
 *
 * $Author: ejt $
 * $Name:  $
 * $Revision: 1.2 $
 * $State: Exp $
 * $Date: 2003/01/09 02:02:59 $
 */

